__doc__ = 'misc functions used by HiperDom'

from ZPublisher.mapply import mapply

def missing_name(name, namespace):
    if name=='self': return namespace.this
    raise TypeError, 'Argument %s not found in namespace' % name

def getObject(name, namespace, template, call=1):
    """Retrieve an object from a given namespace, and call it if needed.
    
    Objects are named by their ID, or their relative or absolute traversable
    path, indicated by slashes in the name.
    
    """

    if '/' in name:
        obj = template.restrictedTraverse(name)
    else:
        obj = namespace.getitem(name, 0)

    # Avoid acquisition when checking for certain aspects
    obj_base = getattr(obj, 'aq_base', obj)
    if not call or not callable(obj_base): return obj

    if hasattr(obj_base, '__render_with_namespace__'):
        obj.__render_with_namespace__(namespace)
    if getattr(obj_base, 'isDocTemp', None):
        return obj(namespace.this, namespace)

    # Everything else uses ZPublisher.mapply.
    if not hasattr(namespace, 'get') and hasattr(namespace, 'getitem'):
        # We 'enhance' the namespace object by adding a 'get' method, so mapply
        # can use it as it uses any other mapping object.
        def namespace_get_method(key, default=None, _ns=namespace):
            try:
                return _ns.getitem(key)
            except KeyError:
                return default

        namespace.get = namespace_get_method
        
    try:
        return mapply(obj, keyword=namespace, maybe=1,
            missing_name=missing_name, context=namespace, bind=1)
    # Sigh. It turns out that callable() always returns 1 for Extension classes.
    # So we compensate with a few well-aimed except clauses.
    except AttributeError:
        # If the object does have a __call__ method, something else went wrong.
        if hasattr(obj_base, '__call__'):
            raise
    except TypeError:
        pass

    return obj
